#!/usr/bin/perl
use strict;
use warnings;
use XML::LibXML;

# BuildRequires perl-XML-LibXML
# Script is for testing! 

my $VERSIJA = '0.01.00α © Karlo no DAC 2025.02.20';
my $LICENCE = 'GPLv3';
my $APRAKSTS = 'Script for fixing TTX font <name> Table entries';
my %enviro;

sub paskaidro {
	&versija;
	print "\n" . '-' x 70  . "\n" .
		' ' x 3 . "$enviro{ColourInfo}-h$enviro{ColourReset}              This explanation.\n" .
		' ' x 3 . "$enviro{ColourInfo}in=DIR1$enviro{ColourReset}         The input directory.\n" .
		' ' x 3 . "$enviro{ColourInfo}out=DIR2$enviro{ColourReset}        The output directory.\n" .
		' ' x 3 . "$enviro{ColourInfo}family=FONTNAME$enviro{ColourReset} The new family name (optional).\n" .
		' ' x 3 . "$enviro{ColourInfo}ID1617$enviro{ColourReset}          Use font name from the <OS_2> Table entries\n" .
		' ' x 3 . "$enviro{ColourInfo}      $enviro{ColourReset}          nameID=16 and nameID=17 (optional).\n" .
		' ' x 3 . "$enviro{ColourInfo}WAF$enviro{ColourReset}             Width as separate family (optional).\n" .
		'-' x 70 . "\n";
	print "\n";
}

sub versija {
	print "\nVersion:\t$VERSIJA\nLicence\t$LICENCE\n$enviro{ColourName}$APRAKSTS$enviro{ColourReset}\n";
}

#
# Script will update TTX font <name> Table:
#   nameID=1  (Family Name)
#   nameID=2  (Subfamily Name),
#   nameID=4  (Full Font Name,
#   nameID=6  (PostScript Name)
#   nameID=16 (Typographic Family Name)
#   nameID=17 (Typographic Subfamily Name)
# based on extracted and provided values.
#

# Parse <OS_2> Table:
#    Extract usWeightClass, usWidthClass, and fsSelection values.
#
# If parameter 'ID1617' is used, script forcibly uses values from:
#     nameID=16
#         (Typographic Family Name. Same for all styles.
#          Used for grouping in modern applications.)
# and
#     nameID=17
#        (Typographic Subfamily Name. Used for distinguishing styles.
#         Example: "Regular", "Bold", "Italic". Should matche with nameID=2)

$enviro{ColourAlert} = "\e[91m\e[44m";
$enviro{ColourInfo} = "\e[92m";
$enviro{ColourName} = "\e[93m";
$enviro{ColourReset} = "\e[m";

foreach (@ARGV){
	if ($_ eq '-h' || $_ eq '-help' || $_ eq '--help'){
		&paskaidro;
		exit;
	}
	elsif ($_ eq '-v'){
		&versija;
		exit;
	}
	elsif (/^in=/){
		$enviro{SourceDIR} = substr ($_,3);
	}
	elsif (/^out=/){
		$enviro{DestDIR} = substr ($_,4);
	}
	elsif (/^family=/){
		$enviro{Family} = substr ($_,7);
	}
	elsif (/^ID1617$/){
		$enviro{Typography} = $_;
	}
	elsif (/^WAF$/){
		$enviro{WidthAsFamily} = $_;
	}
	else {
		print "$enviro{ColourInfo} Unknown parameter: $enviro{ColourName}$_$enviro{ColourReset}\n";
	}
}

{
	my $kljuuda=0;
	if (!$enviro{SourceDIR}){
		print "$enviro{ColourAlert} Directory name for input files is missing.$enviro{ColourReset}\n";
		$kljuuda=1;
	}
	elsif (! -d $enviro{SourceDIR}){
		print "$enviro{ColourAlert} The input directory „$enviro{SourceDIR}” does not exist.$enviro{ColourReset}\n" ;
		$kljuuda=1;
	}
	if (!$enviro{DestDIR}){
		print "$enviro{ColourAlert} Directory name for output files is missing. $enviro{ColourReset}\n";
		$kljuuda=1;
	}
	elsif (-d $enviro{DestDIR}){
		$kljuuda = &NotEmpty($enviro{DestDIR});
	}
	else{
		mkdir $enviro{DestDIR} or die("Cannot create output directory „$enviro{DestDIR}”!");
	}
	if ($kljuuda){
		&paskaidro;
		exit 1
	}
}


# Mapping for Weight, Width, and Italic styles
my %weight_name_map = (
    100 => 'Thin', 200 => 'Extra Light', 300 => 'Light', 400 => 'Regular',
    500 => 'Medium', 600 => 'Semi Bold', 700 => 'Bold', 800 => 'Extra Bold', 900 => 'Black'
);

my %width_name_map = (
    1 => 'Ultra-condensed', 2 => 'Extra-condensed', 3 => 'Condensed', 
    4 => 'Semi-condensed', 5 => 'Medium', 6 => 'Semi-expanded', 
    7 => 'Expanded', 8 => 'Extra-expanded', 9 => 'Ultra-expanded'
);

my $italic_flag = 0x001;  # Bit 0 for Italic


# Process each .ttx file in the directory
opendir(DIR, $enviro{SourceDIR}) or die "Cannot open directory: $!";
my @files = grep(/\.ttx$/, readdir(DIR));
closedir(DIR);

foreach my $file (@files) {
	my $file_path = "$enviro{SourceDIR}/$file";
	print "Processing: $file_path\n";

	my $dom = XML::LibXML->load_xml(location => $file_path);

	# Extract <OS_2> values
	my $os2 = $dom->findnodes('//OS_2')->[0];
	my $weight = $os2->findnodes('./usWeightClass')->[0]->getAttribute('value');
	my $width = $os2->findnodes('./usWidthClass')->[0]->getAttribute('value');
	my $fsSelection = $os2->findnodes('./fsSelection')->[0]->getAttribute('value');
	$fsSelection =~ s/\s//g;
	$fsSelection = hex($fsSelection);

	# Determine Style Information
	my ($weight_name, $width_name, $italic_name, $familyname, $stylename, $postscript_name);
	$weight_name = $weight_name_map{$weight} // die "_Wrong_weight_$weight";
	$width_name = $width_name_map{$width} // die "_Wrong_width_$width";
	my $is_italic = ($fsSelection & $italic_flag) ? 1 : 0;
	$italic_name = $is_italic ? 'Italic' : '';

	# Read <name> table entries
	for my $namerecord ($dom->findnodes('//name/namerecord')) {
		my $nameID = $namerecord->getAttribute('nameID');
		if ($nameID == 16) {
			$familyname = $namerecord->to_literal();
			$familyname =~ s/^\s+|\s+$//g;
		}
		elsif ($nameID == 17) {
			$stylename = $namerecord->to_literal();
			$stylename =~ s/^\s+|\s+$//g;
		}
	}

	if ($enviro{Typography} && !$familyname){
		die "The typogrphay name is missing.";
	}

	# Last check:
	die "Missing familyname!" if (!$familyname && !$enviro{Family});

	# Construct Style and Full Names
	# Family Name
	if ($enviro{Family}) {
		$familyname = $enviro{Family};
		if ($enviro{WidthAsFamily}){
			if ($width_name ne 'Medium') {
				$familyname = "$familyname $width_name";
			}
		}
	}
	
	# Style Names
	if ($enviro{WidthAsFamily}){
		$stylename = join(' ', grep {$_} ($weight_name, $italic_name));
	}

	# Width is included into the same family.
	# Not compatible with «Libre Office»!
	else {
	  $stylename = join(' ', grep {$_} ($width_name, $weight_name, $italic_name));
	}
	$postscript_name = $stylename;
	$postscript_name =~ s/\s+//g;

	print "Family: $familyname; Style: $stylename\n";

	# Update <name> table entries
	for my $namerecord ($dom->findnodes('//name/namerecord')) {
		my $nameID = $namerecord->getAttribute('nameID');
		if ($nameID == 1) {
			$namerecord->removeChildNodes();
			$namerecord->appendText($familyname);
		}
		elsif ($nameID == 2) {
			$namerecord->removeChildNodes();
			$namerecord->appendText($stylename);
		}
		elsif ($nameID == 4) {
			$namerecord->removeChildNodes();
			$namerecord->appendText("$familyname $stylename");
		}
		elsif ($nameID == 6) {
			$namerecord->removeChildNodes();
			$namerecord->appendText("$familyname-$postscript_name");
		}
		elsif ($nameID == 16) {
			$namerecord->removeChildNodes();
			$namerecord->appendText($familyname);
		}
		elsif ($nameID == 17) {
			$namerecord->removeChildNodes();
			$namerecord->appendText($stylename);
		}
	}

	# Save modified .ttx file
	my $output_file = "$enviro{DestDIR}/$file";
	open my $out, '>', $output_file or die "Cannot open $output_file: $!";
	print $out $dom->toString(1);
	close $out;
	# print "Adjusted file saved: $output_file\n";
}

sub NotEmpty {
	opendir(DIR,shift) or die $!;
	my @files = grep { !m/\A\.{1,2}\Z/} readdir(DIR);
	closedir(DIR);
	if (@files) {
		print "$enviro{ColourAlert} The output directory „$enviro{DestDIR}” is not empty.$enviro{ColourReset}\n";
		return 1;
	}
	else {
		return 0;
	}
}
